#!/usr/bin/env python
#coding: utf-8

# cli.py - command-line part of speechdesktop
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.


import sys
import speech

def uni(t):
	if isinstance(t,str):
		t = t.decode('utf-8')
	elif not isinstance(t,unicode):
		t = unicode(t)
	return t

def getClipBoardText(mode,speaker):
	engine=speaker.get_conf('common','clipboard',None)
	if engine not in ('qt','pygtk'):
		try:
			from gi.repository import Gtk, Gdk
			m = Gdk.SELECTION_CLIPBOARD if mode == 'C' else Gdk.SELECTION_PRIMARY
			clip = Gtk.Clipboard.get (m)
			text = clip.wait_for_text ()
			return uni(text)
		except:
			pass
	if engine not in ('gtk','qt'):
		try:
			import gtk
			m = gtk.gdk.SELECTION_CLIPBOARD if mode == 'C' else gtk.gdk.SELECTION_PRIMARY
			clip = gtk.clipboard_get (m)
			text = clip.wait_for_text ()
			return uni(text)
		except:
			pass
	if engine not in ('gtk','pygtk'):
		try:
			import PyQt4.QtGui
			app = PyQt4.QtGui.QApplication([])
			cb = PyQt4.QtGui.QApplication.clipboard()
			m = cb.Clipboard if mode == 'C' else cb.Selection
			return uni(cb.text(m))
		except:
			pass
	return None

	
def get_speak_message(command,speaker):
	global args
	if command == 'clip':
		return getClipBoardText('C',speaker)
	if command == 'sel':
		return getClipBoardText('S',speaker)
	if command == 'hour':
		import datetime
		d=datetime.datetime.now()
		return d.strftime('%H:%M:%S')
	if command == 'date':
		import datetime,locale
		import i18n
		clang=i18n.curlang()
		slang=speaker.lang()
		if not clang.startswith(slang):
			clang=i18n.trans(slang,'languages','fullname')
		try:
			locale.setlocale(locale.LC_ALL,clang)
		except:
			try:
				locale.setlocale(locale.LC_ALL,'')
			except:
				pass
		
		d=datetime.datetime.now()
		fmt = '%A, '+locale.nl_langinfo(locale.D_FMT)+', %H:%M'
		return d.strftime(fmt)
	if command == 'line':
		return ' '.join(args[1:])
	print """Usage: %s [queue] <command>
       %s [queue] line text to say
where command may be:
  stop - stop all speech
  clip - say clipboard content
  sel  - say current selection
  hour - say hour
  date - say date""" % (sys.argv[0],sys.argv[0])
	return None


def run():
	command='help'
	force_stop=True
	args=sys.argv[1:]
	if len(args) > 1 and args[0] == 'queue':
		force_stop=False
		args=args[1:]
	if len(args) > 0:
		command=args[0]
	if command == 'stop':
		speech.fastStop()
		sys.exit(0)

	speaker=speech.SpdsCore()

	try:
		to_say = get_speak_message(command,speaker)
	except SystemExit:
		exit(0)
	except:
		import traceback
		traceback.print_exc()
		to_say = None
	speaker.say(to_say, force_stop)
	speaker.close()
