#!/usr/bin/env python
#coding: utf-8

# gtkgui.py - Gtk-3.0 GUI part of speechdesktop
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.


import dbus,sys,getopt
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import Gtk, Gdk, GdkPixbuf
import speech,pidgin

speaker=None

def berror():
	global speaker
	import traceback
	traceback.print_exc()
	if speaker:
		speaker.close()
	sys.exit(0)

def init_dbus_listeners():
	global speaker
	dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
	bus = dbus.SessionBus()
	gaim=pidgin.Pidgin(bus,speaker)
	
class GtkGui(object):
	
	def __init__(self,speaker,noicon,tray):
		self.noicon=noicon
		self.speaker=speaker
		self._sensitive=False
		self.window = None
		self.tray=tray
		self._modules=None
		self._voices=None
		self._last_icon_path=None
		if self.noicon:
			self.show()
	
	def alert(self,txt):
		dialog=Gtk.MessageDialog(self.window,
			Gtk.DialogFlags.DESTROY_WITH_PARENT,
			Gtk.MessageType.INFO,
			Gtk.ButtonsType.CLOSE,
			txt)
		dialog.run()
		dialog.destroy()
			
	
	def _create_window(self):
		if not self.tray or not self.tray.is_embedded():
			self.noicon=True
		self.window = Gtk.Window()
		self.window.set_title('Speaking Desktop Preferences')
		vbox=Gtk.Box.new(Gtk.Orientation.VERTICAL,2)
		self.window.add(vbox)

		# upper: Speech Dispatcher

		frame=Gtk.Frame.new("Speech-Dispatcher")
		vbox.pack_start(frame,0,0,0)
		grid=Gtk.Grid()
		frame.add(grid)
	
		grid.attach(Gtk.Label.new("Module"),0,0,1,1)
		grid.attach(Gtk.Label.new("Voice"),2,0,1,1)
		grid.attach(Gtk.Label.new("Rate"),0,1,1,1)
		grid.attach(Gtk.Label.new("Pitch"),0,2,1,1)
		grid.attach(Gtk.Label.new("Volume"),0,3,1,1)
		self.combo_vm=Gtk.ComboBoxText()
		grid.attach(self.combo_vm,1,0,1,1)
		self.combo_vv=Gtk.ComboBoxText()
		grid.attach(self.combo_vv,3,0,1,1)
		
		self.combo_vm.connect("changed",self.module_changed)
		self.combo_vv.connect("changed",self.voice_changed)
		
		self.range_rt=Gtk.HScale.new_with_range(-100.0,100.0,1)
		grid.attach(self.range_rt,1,1,3,1)
		self.range_rt.connect("value-changed",self.set_rate)

		self.range_pi=Gtk.HScale.new_with_range(-100.0,100.0,1)
		grid.attach(self.range_pi,1,2,3,1)
		self.range_pi.connect("value-changed",self.set_pitch)

	
		self.range_vol=Gtk.HScale.new_with_range(-100.0,100.0,1)
		grid.attach(self.range_vol,1,3,3,1)
		self.range_vol.connect("value-changed",self.set_volume)
		
		but=Gtk.Button.new_with_label('Test')
		grid.attach(but,0,4,1,1)
		but.connect("pressed",self.say_test)

		but=Gtk.Button.new_with_label('Stop')
		grid.attach(but,3,4,1,1)
		but.connect("pressed",self.say_stop)
		
		self.tentry=Gtk.Entry.new()
		grid.attach(self.tentry,1,4,2,1)

		# next: Pidgin
		
		frame=Gtk.Frame.new("Pidgin")
		vbox.pack_start(frame,0,0,0)
		grid=Gtk.Grid()
		frame.add(grid)
		grid.attach(Gtk.Label.new("Messages"),0,0,1,1)
		self.switch_msg=Gtk.Switch()
		self.switch_msg.connect("notify::active",self.simple_changer,'pidgin','message')
		grid.attach(self.switch_msg,1,0,1,1)
		grid.attach(Gtk.Label.new("Announces"),2,0,1,1)
		self.switch_ann=Gtk.Switch()
		self.switch_ann.connect("notify::active",self.simple_changer,'pidgin','announce')
		grid.attach(self.switch_ann,3,0,1,1)
		

		#next: icon
		
		frame=Gtk.Frame.new("Tray icon")
		vbox.pack_start(frame,0,0,0)
		box=Gtk.Box.new(Gtk.Orientation.HORIZONTAL,2)
		frame.add(box)
		self.IconButton=Gtk.Button.new()
		box.pack_start(self.IconButton,0,0,0)
		self.IconButton.connect("pressed",self.select_icon)
		but=Gtk.Button.new_with_label('Clean')
		box.pack_start(but,0,0,0)
		but.connect("pressed",self.deselect_icon)
		
		

		#next: buttons
		
		bbox=Gtk.ButtonBox.new(Gtk.Orientation.HORIZONTAL)
		vbox.pack_start(bbox,0,0,0)
		but1=Gtk.Button.new_from_stock(Gtk.STOCK_SAVE)
		bbox.pack_start(but1,0,0,0)
		but2=Gtk.Button.new_from_stock(Gtk.STOCK_CANCEL)
		bbox.pack_start(but2,0,0,0)
		if not self.noicon:
			but3=Gtk.Button.new_from_stock(Gtk.STOCK_CLOSE)
			bbox.pack_start(but3,0,0,0)
			but3.connect("pressed",self.close)
		but4=Gtk.Button.new_from_stock(Gtk.STOCK_QUIT)
		bbox.pack_start(but4,0,0,0)
		but4.connect("pressed",self.quit)
		but1.connect("pressed",self.spd_save)
		but2.connect("pressed",self.spd_revert)
		self.window.connect("delete-event",self.onclose)
			

	def say_test(self,*args):
		self.speaker.say(self.tentry.get_text())
	
	def say_stop(self,*args):
		self.speaker.stop()
		
	def quit(self,*args):
		Gtk.main_quit()

	def close(self,*args):
		self.speaker.set_conf('gui','test-text',self.tentry.get_text())
		self.window.hide()
		
	def onclose(self,*args):
		if self.noicon:
			Gtk.main_quit()
			return False
		else:
			self.speaker.set_conf('gui','test-text',self.tentry.get_text())
			self.window.hide()
			return True
	
	def set_rate(self,*args):
		if not self._sensitive:
			return
		rate=int(self.range_rt.get_value())
		self.speaker.set_rate(rate,True)

	def set_pitch(self,*args):
		if not self._sensitive:
			return
		pitch=int(self.range_pi.get_value())
		self.speaker.set_pitch(pitch,True)

	def set_volume(self,*args):
		if not self._sensitive:
			return
		vol=int(self.range_vol.get_value())
		self.speaker.set_volume(vol,True)


	def module_changed(self,*args):
		if not self._sensitive:
			return
		if not self._modules:
			return
		nr=self.combo_vm.get_active()
		txt=self._modules[nr]
		self.speaker.set_module(txt)
		self.speaker.set_conf('speechd','module',txt)
		self._sensitive=False
		self.init_voice_combo()
		self._sensitive=True
		nr=self.combo_vv.get_active()
		txt=self._voices[nr][0]
		lang=self._voices[nr][1]
		self.speaker.set_voice(txt,lang)
		self.speaker.set_conf('speechd','voice',txt)
		
	
	def voice_changed(self,*args):
		if not self._sensitive:
			return
		if not self._voices:
			return
		nr=self.combo_vv.get_active()
		txt=self._voices[nr][0]
		lang=self._voices[nr][1]
		self.speaker.set_voice(txt,lang)
		self.speaker.set_conf('speechd','voice',txt)
		
		
	def simple_changer(self,widget,param,section,key):
		if not self._sensitive:
			return
		t='t' if widget.get_active() else 'f'
		self.speaker.set_conf(section,key,t)
		
	def spd_save(self,*args):
		self.speaker.save_conf()
		self.alert("OK")
	
	def spd_revert(self,*args):
		self.speaker.refresh(True)
		self.restore_buttons()
	
	def init_voice_combo(self):
		self._voices=self.speaker.get_voices()
		if not self._voices:
			return
		voice=self.speaker.get_conf('speechd','voice')
		self.combo_vv.remove_all()
		nr=0
		for n,m in enumerate(self._voices):
			laa=m[1] or 'none'
			self.combo_vv.append_text(m[0]+' ('+laa+')')
			if m[0] == voice:
				nr=n
		self.combo_vv.set_active(nr)
	
	def init_module_combo(self):
		self._modules=self.speaker.get_modules()
		if self._modules:
			module=self.speaker.get_conf('speechd','module')
			nr=0
			self.combo_vm.remove_all()
			for n,m in enumerate(self._modules):
				self.combo_vm.append_text(m)
				if m == module:
					nr=n
			self.combo_vm.set_active(nr)
			self.init_voice_combo()
		
	def update_icon_pv(self,dialog,image,label):
		path=dialog.get_filename()
		pixbuf=None
		width=0
		height=0
		try:
			pixbuf=GdkPixbuf.Pixbuf.new_from_file(path)
			width=pixbuf.get_width()
			height=pixbuf.get_height()
			if width > 64 or height > 64:
				mpw=64.0/width
				mph=64.0/height
				if mpw < mph:
					mpw=mph
				pixbuf=pixbuf.scale_simple(int(mpw*width),int(mpw*height),2)
		except:
			pixbuf=None
			pass
		if pixbuf:
			image.set_from_pixbuf(pixbuf)
		label.set_text('%d x %d' % (width,height))
		
	def select_icon(self,*args):
		dialog=Gtk.FileChooserDialog('Select icon',
			self.window,
			Gtk.FileChooserAction.OPEN)
		dialog.add_buttons(
			Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,
			Gtk.STOCK_OPEN,Gtk.ResponseType.OK)
		f1=Gtk.FileFilter()
		f1.set_name('All files')
		f1.add_pattern('*')
		dialog.add_filter(f1)
		f2=Gtk.FileFilter()
		f2.set_name('Images')
		f2.add_pixbuf_formats()
		dialog.add_filter(f2)
		dialog.set_filter(f2)
		dialog.set_local_only(True)
		
		preview=Gtk.Box.new(Gtk.Orientation.VERTICAL,2)
		image=Gtk.Image()
		preview.pack_start(image,0,0,0)
		label=Gtk.Label('')
		preview.pack_start(label,0,0,0)
		preview.show_all()
		dialog.set_preview_widget(preview)
		dialog.set_use_preview_label(False)
		dialog.connect('update-preview',self.update_icon_pv,image,label)
		
		path=self.speaker.get_conf('gui','iconfile')
		if not path:
			path=self._last_icon_path
		if path:
			dialog.set_filename(path)
		rc=dialog.run()
		path=None
		if rc == Gtk.ResponseType.OK:
			path=dialog.get_filename()
		dialog.destroy()
		if rc == Gtk.ResponseType.OK:
			self.speaker.set_conf('gui','iconfile',path)
			self.set_tray()
			self.make_icon_button()
	
	def deselect_icon(self,*args):
		path=self.speaker.get_conf('gui','iconfile')
		if path:
			self._last_icon_path=path
		self.speaker.set_conf('gui','iconfile','')
		self.set_tray()
		self.make_icon_button()
		
	def set_tray(self):
		pixbuf=None
		pline=self.speaker.get_conf('gui','iconfile')
		if pline:
			try:
				pixbuf=GdkPixbuf.Pixbuf.new_from_file(pline)
			except:
				pass
		if pixbuf:
			self.tray.set_from_pixbuf(pixbuf)
		else:
			self.tray.set_from_stock(Gtk.STOCK_PREFERENCES)

	def make_icon_button(self):
		pixbuf=None
		pline=self.speaker.get_conf('gui','iconfile')
		if pline:
			try:
				pixbuf=GdkPixbuf.Pixbuf.new_from_file(pline)
				width=pixbuf.get_width()
				height=pixbuf.get_height()
				if width > 64 or height > 64:
					mpw=64.0/width
					mph=64.0/height
					if mpw < mph:
						mpw=mph
					pixbuf=pixbuf.scale_simple(int(mpw*width),int(mpw*height),2)
			except:
				pixbuf=None
		if pixbuf:
			img=Gtk.Image.new_from_pixbuf(pixbuf)
		else:
			img=Gtk.Image.new_from_stock(Gtk.STOCK_PREFERENCES,Gtk.IconSize.BUTTON)
		self.IconButton.set_image(img)

	def restore_buttons(self):
		self._sensitive=False
		self.init_module_combo()
		self.range_rt.set_value(float(self.speaker.get_conf_i('speechd','rate',0)))
		self.range_pi.set_value(float(self.speaker.get_conf_i('speechd','pitch',0)))
		self.range_vol.set_value(float(self.speaker.get_conf_i('speechd','volume',0)))
		self.switch_msg.set_active(self.speaker.get_conf_b('pidgin','message'))
		self.switch_ann.set_active(self.speaker.get_conf_b('pidgin','announce'))
		self.tentry.set_text(self.speaker.get_conf('gui','test-text','Test'))
		self.make_icon_button()
		self.set_tray()
		self._sensitive=True
		
	def show(self,*args):
		if not self.window:
			self._create_window()
		try:
			self.restore_buttons()
			self.window.show_all()
		except:
			import traceback
			traceback.print_exc()
			self.speaker.close()
			Gtk.main_quit()
			sys.exit(1)

def helpme():
	print """Usage: %s [-h] | [-n]
  -h - this help
  -n - no tray icon""" % sys.argv[0]
	exit(1)

def runme():
	global speaker
	opts, args= getopt.getopt(sys.argv[1:],"hn")
	noicon=False
	for opt in opts:
		if opt[0] == '-h':
			helpme()
		if opt[0] == '-n':
			noicon=True

	speaker=speech.SpdsCore()
	try:
		trayIcon=None
		if not noicon:
			iconpic=speaker.get_conf('gui','iconfile',None)
			pixbuf=None
			trayIcon=Gtk.StatusIcon.new_from_stock(Gtk.STOCK_PREFERENCES)
			if iconpic:
				try:
					pixbuf=GdkPixbuf.Pixbuf.new_from_file(iconpic)
				except:
					pass
				if pixbuf:
					trayIcon.set_from_pixbuf(pixbuf)
			trayIcon.set_title(u"SPD")
			trayIcon.set_tooltip_text(u"Speaking Desktop Preferences");
		gui=GtkGui(speaker,noicon,trayIcon)
		init_dbus_listeners()
		if trayIcon:
			trayIcon.connect("activate",gui.show)
	except:
		berror()
	
	Gtk.main()
	speaker.close()

