#!/usr/bin/env python2.7
#coding: utf-8

# i18n.py - internationalization of speechdesktop
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.


import locale,os,sys

def curlang():
	lang=locale.setlocale(locale.LC_ALL)
	if lang != 'C':
		return lang
	pipe=os.pipe()
	pid=os.fork()
	if pid < 0:
		os.close(pipe[0])
		os.close(pipe[1])
		return 'C'
	if pid == 0:
		os.close(pipe[0])
		try:
			lang=locale.setlocale(locale.LC_ALL,"")
		except:
			lang='en'
		os.write(pipe[1],lang)
		os.close(pipe[1])
		sys.exit(0)
	os.waitpid(pid,0)
	os.close(pipe[1])
	lang=os.read(pipe[0],100)
	os.close(pipe[0])
	return lang


# Translation must not be done with gettext, because locale defined
# language may be not the same as synthesizer language!

# each key must be present in 'en' subsections!
# en subsection must be present in each section!

_transt={
	'pidgin': {
		'en':{
			'buddyin':u'%s is online',
			'buddyout':u'%s goes offline',
			'message':u'%s says: %s'
			},
		'pl': {
			'buddyin':u'%s połączył się',
			'buddyout':u'%s rozłączył się',
			'message':u'%s pisze: %s'
			}
		},
		'languages': {
			'en': {
				'fullname' : 'en_US.UTF-8'
			},
			'pl': {
				'fullname': 'pl_PL.UTF-8'
			}
		}	  
	}
	
def trans(lang,section,key):
	global _transt
	if not section in _transt:
		return key
	s=_transt[section]
	if not lang in s:
		lang='en'
	t=s[lang].get(key,None)
	if not t:
		t=s[lang].get(key,key)
	return t

